#define F_CPU 12000000

#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>

#define AREF 3.3


#define TCS_S2_DDR DDRA
#define TCS_S2_PORT PORTA
#define TCS_S2_POS PA6

//S3 Connection
#define TCS_S3_DDR DDRA
#define TCS_S3_PORT PORTA
#define TCS_S3_POS PA1

//Out Connection
#define TCS_OUT_DDR DDRA
#define TCS_OUT_PORT PINA
#define TCS_OUT_POS PA2

//LED Connection
#define TCS_LED_DDR DDRA
#define TCS_LED_PORT PORTA
#define TCS_LED_POS PA3


#define	TCSS2High()	TCS_S2_PORT|=(1<<TCS_S2_POS)
#define	TCSS2Low()	TCS_S2_PORT&=~(1<<TCS_S2_POS)
#define	TCSS3High()	TCS_S3_PORT|=(1<<TCS_S3_POS)
#define	TCSS3Low()	TCS_S3_PORT&=~(1<<TCS_S3_POS)
#define TCSLEDOn()	TCS_LED_PORT|=(1<<TCS_LED_POS)
#define TCSLEDOff() TCS_LED_PORT&=~(1<<TCS_LED_POS)

int ADC_value;

void ADC_init(void)
{
    ADMUX = 0;
	ADMUX |= (1 << MUX0);          // ADC1 - channel 1
	ADMUX |= (1 << REFS0);         // AVCC with external capacitor at AREF pin
	ADCSRA = 0;
	ADCSRA |= (5 << ADPS0);
	ADCSRA = (1 << ADEN);
}

int ADC_get(uint8_t channel)
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

double ADC_voltage(int raw)
{
    (void)raw;
    return raw * AREF / 1024.;
}

void InitTCS3200()
{
	//Initialize S2 and S3 line as OUTPUT
	TCS_S2_DDR|=(1<<TCS_S2_POS);
	TCS_S3_DDR|=(1<<TCS_S3_POS);
	
	//LED Port
	TCS_LED_DDR|=(1<<TCS_LED_POS);	
	
	DDRA |= (1 << PA4);
	DDRA &= ~(1 << PA5);
}


void TCSSelectRed()
{
	TCSS2Low();
	TCSS3Low();
}

void TCSSelectGreen()
{
	TCSS2High();
	TCSS3High();
}

void TCSSelectBlue()
{
	TCSS2Low();
	TCSS3High();
}

void TCSSelectClear()
{
	TCSS2High();
	TCSS3Low();
}

uint32_t TCSMeasure()
{
   //If the function is entered when the level on OUT line was low
   //Then wait for it to become high.
   if(!(TCS_OUT_PORT & (1<<TCS_OUT_POS)))
   {
      while(!(TCS_OUT_PORT & (1<<TCS_OUT_POS)));   //Wait for rising edge  
   }


   while(TCS_OUT_PORT & (1<<TCS_OUT_POS));   //Wait for falling edge

   TCNT1=0x0000;//Reset Counter

   TCCR1B=(1<<CS10); //Prescaller = F_CPU/1 (Start Counting)

   while(!(TCS_OUT_PORT & (1<<TCS_OUT_POS)));   //Wait for rising edge

   //Stop Timer
   TCCR1B=0x00;

   return ((float)8000000UL/TCNT1);

}

uint32_t MeasureR()
{
	TCSSelectRed();
	uint32_t r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/3.3;
	
}

uint32_t MeasureG()
{
	TCSSelectGreen();
	uint32_t r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/3;
	
}

uint32_t MeasureB()
{
	TCSSelectBlue();
	uint32_t r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/4.2;
	
}

int main(void)
{

	

	ADC_init();
	
	double lum_val;

	
	//Initialize TCS Library
	InitTCS3200();
	

	DDRD |= (1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3)
			| (1 << PD4) | (1 << PD5);
	DDRC = 0xff;
	
	
	while(1)
	{	
		////  LIGHT INTENSITY  ////
		
		lum_val = ADC_get(0);
		
		PORTC &= ~(1 << PC0) & ~(1 << PC1) & ~(1 << PC2)
				& ~(1 << PC3) & ~(1 << PC4); //disable white LEDs
		
		if(lum_val < 500)
		{
			PORTC |= (1 << PC0);
			if(lum_val < 400)
			{
				PORTC |= (1 << PC1);
				if(lum_val < 300)
				{
					PORTC |= (1 << PC2);
					if(lum_val < 200)
					{
						PORTC |= (1 << PC3);
						if(lum_val < 100)
						{
							PORTC |= (1 << PC4);
						}
					}
				}
			}	
		}
		
		///////////////////////////
		
		
		////  LIGHT COLOR  ////
		
		uint32_t r,g,b;
	
		TCSLEDOn();
		
		r=MeasureR();
		g=MeasureG();
		b=MeasureB();
		
		TCSLEDOff();
		
		uint32_t smallest;
	
		if(r<b)
		{
			if(r<g)
				smallest=r;
			else
				smallest=g;
		}
		else
		{
			if(b<g)
				smallest=b;
			else
				smallest=g;	
		}
	
		uint32_t _r,_g,_b;
		smallest=smallest/10;
	
		_r=r/smallest;
		_g=g/smallest;
		_b=b/smallest;
		
		uint32_t max = (_r > _b ? (_r > _g ? _r : _g) : (_b > _g ? _b : _g));
		uint32_t min = (_r < _b ? (_r < _g ? _r : _g) : (_b < _g ? _b : _g));
		uint32_t div = (max - min) / 3;
		
		
		PORTD &= ~(1 << PD0) & ~(1 << PD1) & ~(1 << PD2)
				& ~(1 << PD3) & ~(1 << PD4) & ~(1 << PD5);
		PORTC &= ~(1 << PC5) & ~(1 << PC6) & ~(1 << PC7);
		
		if(_r > min)
		{
			PORTD |= (1 << PD0);  //R1 on
			if(_r > min + div)
			{
				PORTD |= (1 << PD3);  //R2 on
				if(_r > min + 2 * div)
					PORTC |= (1 << PC5);  //R3 on
			}
		}
		
		if(_g > min)
		{
			PORTD |= (1 << PD1);  //G1 on
			if(_g > min + div)
			{
				PORTD |= (1 << PD4);  //G2 on
				if(_g > min + 2 * div)
					PORTC |= (1 << PC6);  //G3 on
			}
		}
		
		if(_b > min)
		{
			PORTD |= (1 << PD2);  //B1 on
			if(_b > min + div)
			{
				PORTD |= (1 << PD5);  //B2 on
				if(_b > min + 2 * div)
					PORTC |= (1 << PC7);  //B3 on
			}
		}
		
		/*
		if(_r > _b && _r > _g)
		{
			PORTD |= (1 << PD0);  //R on
			PORTD &= ~(1 << PD1); //G off
			PORTD &= ~(1 << PD2); //B off
			}
		else
		{
			if(_g > _b)
			{
				PORTD &= ~(1 << PD0); //R off
				PORTD |= (1 << PD1);  //G on
				PORTD &=~ (1 << PD2); //B off
			}	
			else
			{
				PORTD &= ~(1 << PD0); //R off
				PORTD &= ~(1 << PD1); //G off
				PORTD |= (1 << PD2);  //B on
			}
		}
		*/		
		///////////////////////////
		
		_delay_ms(50);
	}
	
}
